
//////////////////////////////////////////////////////////////////////////////
//																			//
// DebugOut.h - Set of macros and routines for debugging purposes			//
//																			//
// Author: 	David Rokhvarg  <davidr@sangoma.com>							//
//																			//
// ======================================================================== //
// April 29, 2001	David Rokhvarg	Initial version.						//
// ************************************************************************ //
//////////////////////////////////////////////////////////////////////////////

#ifndef _DEBUG_OUT_H
#define _DEBUG_OUT_H

#include <stdio.h>
#include <stdarg.h>
#include "public.h"	//for *_DRIVER_NAME

/* Note : PROGRAM_NAME or DRIVER_NAME should be defined before including this file.
e.g. #define PROGRAM_NAME "MyProgram : " 
**/

#define MAX_DEBUG_STRING 500

#if defined(__KERNEL__)

/* These are defined in "sources" file of each driver */
#if defined( BUSENUM_DRV )
# define DRIVER_NAME	SNGBUS_DRIVER_NAME
#elif defined( VIRTUAL_IF_DRV )
# define DRIVER_NAME	SDLADRV_DRIVER_NAME
#elif defined( SPROTOCOL )
# define DRIVER_NAME	SPROTOCOL_DRIVER_NAME
#elif defined( NDIS_MINIPORT_DRIVER )
# define DRIVER_NAME	WANPIPE_DRIVER_NAME
#elif defined( DRVLIB )
/* a library, not a driver, does not have name accessable from user-mode */
# define DRIVER_NAME	"DRVLIB"
/* a library, not a driver, does not have name accessable from user-mode */
#elif defined( BUILD_SWDM_LIB )
# define DRIVER_NAME	"SWDM_LIB"
#endif


#define splimp() 0
#define splx(l)

#define	ERR_DBG_OUT			if(0)DbgPrint
#define DBG_NOT_IMPL		if(0)DbgPrint
#define FUNC_NOT_IMPL()		if(1)DbgPrint("%s()-Not Implemented(File:%s, Line:%d)\n", __FUNCTION__, __FILE__, __LINE__)
#define DBG_DSL_NOT_IMPLD()	FUNC_NOT_IMPL()

/* debugging of SngBus.sys */
#define DEBUG_SNGBUS	if(0)DbgPrint

/* debugging of wanpipe_kernel.h */
#define DBG_KRN			if(0)DbgPrint
#define DBG_COPY_FROM_TO_USER if(0)DbgPrint

/* sdladrv.sys */
#define DBG_ADSL_RX		if(0)DbgPrint
#define DBG_ADSL_TX		if(0)DbgPrint
#define DBG_ADSL_FAST_TX if(0)DbgPrint
#define DBG_ADSL_INIT	if(0)DbgPrint
#define DBG_ADSL_MACADDR if(0)DbgPrint

#define DBG_SET_CFG		if(0)DbgPrint
#define DBG_A200_INIT	if(0)DbgPrint
#define DBG_LIP_OOB		if(0)DbgPrint
#define DEBUG_AFT		if(0)DbgPrint
#define DBG_FAST_TX		if(0)DbgPrint
#define DBG_SHUTDOWN	if(0)DbgPrint

/* sprotocol.sys */
#define DEBUG_LIP		if(0)DbgPrint
#define DBG_LIP_TX		if(0)DbgPrint
#define DBG_LIP_CARRIER	if(0)DbgPrint
#define DEBUG_UDP_SPROT if(0)DbgPrint
#define DBG_LIPAPI		if(0)DbgPrint
#define DBG_LAPD_CFG	if(0)DbgPrint
#define DEBUG_LIPAPI	if(0)DbgPrint

/* wanpipe.sys */
#define DEBUG_REQUEST	if(0)DbgPrint
#define DEBUG_IF_TX		if(0)DbgPrint
#define DEBUG_IF_RX		if(0)DbgPrint
#define DEBUG_IF_STATS	if(0)DbgPrint
#define DEBUG_COMMON	if(0)DbgPrint
#define DEBUG_NET_IF	if(0)DbgPrint

/* drvlib.lib */
#define DBG_BUFLEN		if(0)DbgPrint
#define DBG_GET_REGISTRY if(0)DbgPrint
#define DBG_SPAN_CFG	if(0)DbgPrint

#define ADSL_TEST 0

#define DBG_BUFFER_LEN	512

static void my_func_dbg(char *drv_name, char *func, char *file, int line)
{
	DbgPrint("%s:%s(): File: %s, Line: %d.\n", drv_name, func, file, line);
}

/*#define AFT_FUNC_DEBUG()	if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)*/
#define TDM_FUNC_DBG()		if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define EC_FUNC_DEBUG()		if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define DBG_SET_CFG_FUNC()	if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define DBG_ACUAPI_FUNC()	if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define FUNC_DEBUG()		if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define TDM_FUNC_DBG()		if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define SKB_FUNC()			if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define PROT_FUNC_DEBUG()	if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define TDMAPI_FUNC_DEBUG()	if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)
#define LIP_FUNC()			if(0)my_func_dbg(DRIVER_NAME, __FUNCTION__, __FILE__, __LINE__)


#else //__KERNEL__

////////////////////////////////////////////////////////////////////////////////////////////////////
//redirect debug output to a file, only for user mode applications
static void print_to_file(char *file_name, char *log_string)
{
	char file_path[MAX_PATH];
	FILE *stream;

	GetSystemDirectory(file_path, MAX_PATH);

	_snprintf(&file_path[strlen(file_path)], MAX_PATH - strlen(file_path), "\\%s", 
		file_name);

	stream = fopen(file_path, "a");
	if(stream == NULL){
		OutputDebugString("Failed to open log file!!\n");
		return;
	}

	fprintf(stream, "%s", log_string );
	fflush(stream);
	fclose (stream);
}

static FILE* open_wanpipelog_file()
{
	char wanpipelog_file_path[MAX_PATH];
	char system_windows_dir[MAX_PATH];
	FILE *stream;
	int rc;
#if WINVER < 0x0500
	/* for VC++ 6.0 */
	rc = GetWindowsDirectory(system_windows_dir, MAX_PATH);
#else
	rc = GetSystemWindowsDirectory(system_windows_dir, MAX_PATH);
#endif
	if(rc == 0 || rc > MAX_PATH){
		OutputDebugString("open_wanpipelog_file(): Error: GetSystemWindowsDirectory() Failed!\n");
		return NULL;
	}

	_snprintf(wanpipelog_file_path, MAX_PATH, "%s\\System32\\drivers\\wanpipelog.txt", system_windows_dir);

	stream = fopen(wanpipelog_file_path, "a");
	if(stream == NULL){
		OutputDebugString("open_wanpipelog_file(): Error: Failed to open file!\n");
		return NULL;
	}

	return stream;
}

//print ONLY to wanpipelog.txt
static void _print_to_wanpipelog(void *pszFormat, ...)
{
	FILE *wanpipelog_file = open_wanpipelog_file();
	char str[MAX_DEBUG_STRING];
	va_list	vaArg;

	if(wanpipelog_file == NULL){
		return;
	}
	
	va_start (vaArg, pszFormat);
	_vsnprintf((char*)str, sizeof(str), (char *)pszFormat, vaArg);
	va_end (vaArg);

	fprintf(wanpipelog_file, str); /* print to log to file only */
	fflush(wanpipelog_file);
	fclose (wanpipelog_file);
}

//print to BOTH stdout and wanpipelog.txt
static void print_to_wanpipelog(void *pszFormat, ...)
{
	char str[MAX_DEBUG_STRING];
	va_list	vaArg;

	va_start (vaArg, pszFormat);
	_vsnprintf((char*)str, sizeof(str), (char *)pszFormat, vaArg);
	va_end (vaArg);

	/* print to screen */
	printf(str);

	/* and log to file too */
	_print_to_wanpipelog(str); 
}


#endif

////////////////////////////////////////////////////////////////////////////////////////////////////

//Print "program name" and a line of text.
static void Debug(unsigned int dbg_flag, void *pszFormat, ...)
{
#if 0
#if defined(__KERNEL__)
	DbgPrint("%s():dbg_flag:0x%X\n", __FUNCTION__, dbg_flag);		
#else
	{
		static char dbgtmp[200];
		_snprintf(dbgtmp, 200, "%s(): dbg_flag:0x%X", __FUNCTION__, dbg_flag);
		OutputDebugString(dbgtmp);
		OutputDebugString( "\n" );
	}
#endif
#endif

	if(dbg_flag)
	{	static unsigned char str[MAX_DEBUG_STRING];
		va_list	vaArg;

		va_start (vaArg, pszFormat);
		_vsnprintf((char *)str, MAX_DEBUG_STRING, (char *)pszFormat, vaArg);
		va_end (vaArg);

#if defined(__KERNEL__)
		DbgPrint("%s:", DRIVER_NAME);		
		DbgPrint(str);
#else
#if defined(REDIRECT_TO_FILE)
//#define FILE_NAME	"sangomalog.txt"
		print_to_file(FILE_NAME, (char *)str);
#elif defined(REDIRECT_TO_STDOUT)
		printf((char *)str);
#else
		//to debugger
		OutputDebugString(  PROGRAM_NAME );
		OutputDebugString( (char *)str );
		OutputDebugString( "\n" );
#endif//REDIRECT_TO_FILE
#endif//__KERNEL__
	}//if()
}

//Print a Line of text only, no "program name".
static void DebugL(unsigned char dbg_flag, void *pszFormat, ...)
{
	if(dbg_flag)
	{	static unsigned char	str[MAX_DEBUG_STRING];
		va_list	vaArg;

		va_start (vaArg, pszFormat);
		_vsnprintf((char *)str, MAX_DEBUG_STRING, (char *)pszFormat, vaArg);
		va_end (vaArg);

#if defined(__KERNEL__)
		DbgPrint(str);
#else
		OutputDebugString( (char *)str );
#endif
	}
}

#if !defined(__KERNEL__)

static void TIME_STAMP(UCHAR dbg_flag)
{	SYSTEMTIME st;	
	static char Time_Msg[400];
					
	GetLocalTime(&st);
	_snprintf(Time_Msg, 400, "\n\nDate: Y:%d, M:%d, D:%d. Time: H:%d, M:%d, S:%d\n",
		st.wYear, 
		st.wMonth, 
		st.wDay,
		st.wHour, 
		st.wMinute,
		st.wSecond);
	Debug(dbg_flag, Time_Msg);
}		

//print to console a line WITH program name
static void prn(unsigned int dbg_flag, void *pszFormat, ...)
{
	if(dbg_flag)
	{	static unsigned char str[MAX_DEBUG_STRING];
		va_list	vaArg;

		va_start (vaArg, pszFormat);
		_vsnprintf((char *)str, MAX_DEBUG_STRING, (char *)pszFormat, vaArg);
		va_end (vaArg);

		printf(PROGRAM_NAME);
		printf((char *)str);
	}
}

//print to console a line WITHOUT program name
static void prnl(unsigned int dbg_flag, void *pszFormat, ...)
{
	if(dbg_flag)
	{	static unsigned char str[MAX_DEBUG_STRING];
		va_list	vaArg;

		va_start (vaArg, pszFormat);
		_vsnprintf((char *)str, MAX_DEBUG_STRING, (char *)pszFormat, vaArg);
		va_end (vaArg);

		printf((char *)str);
	}
}


#if defined(_DEBUG) || defined(_MYDEBUG)
////////////////////////////////////////////////////////////////////////////////////////////////////
//Old-style debugging macros to compile legacy code. Do NOT use in any new code.
//
#define Debug1( DBG, _s1 )			{											\
										if(DBG)									\
										{	OutputDebugString( PROGRAM_NAME );	\
											OutputDebugString( _s1 );			\
											OutputDebugString( "\n" );			\
										}										\
									}

#define Debug2( DBG, _s1, _s2 )		{												\
										if(DBG)										\
										{	char	szTempDebugString[MAX_PATH];	\
											OutputDebugString( PROGRAM_NAME );		\
											sprintf( szTempDebugString, _s1, _s2 );	\
											OutputDebugString( szTempDebugString );	\
											OutputDebugString( "\n" );				\
										}											\
									}

#define Debug3( DBG, _s1, _s2, _s3 ){														\
										if(DBG)												\
										{	char	szTempDebugString[MAX_PATH];			\
											OutputDebugString( PROGRAM_NAME );				\
											sprintf( szTempDebugString, _s1, _s2, _s3 );	\
											OutputDebugString( szTempDebugString );			\
											OutputDebugString( "\n" );						\
										}													\
									}

#define Debug4( DBG, _s1, _s2, _s3, _s4 ){														\
										if(DBG)													\
										{	char	szTempDebugString[MAX_PATH];				\
											OutputDebugString( PROGRAM_NAME );					\
											sprintf( szTempDebugString, _s1, _s2, _s3, _s4  );	\
											OutputDebugString( szTempDebugString );				\
											OutputDebugString( "\n" );							\
										}														\
									}

#define Debug5( DBG, _s1, _s2, _s3, _s4, _s5 ){													\
									if(DBG)														\
									{	char	szTempDebugString[MAX_PATH];					\
										OutputDebugString( PROGRAM_NAME );						\
										sprintf( szTempDebugString, _s1, _s2, _s3, _s4, _s5 );	\
										OutputDebugString( szTempDebugString );					\
										OutputDebugString( "\n" );								\
									}															\
								}
#else

#define Debug1( DBG, _s1 )
#define Debug2( DBG, _s1, _s2 )
#define Debug3( DBG, _s1, _s2, _s3 )
#define Debug4( DBG, _s1, _s2, _s3, _s4 )
#define Debug5( DBG, _s1, _s2, _s3, _s4, _s5 )

#endif//#ifdef _DEBUG

#endif//#if !defined(__KERNEL__)
#endif//_DEBUG_OUT_H
